@@ -36,9 +36,8 @@ def test_exact_sequence():
 
 def test_required():
     """Verify that Required works."""
-    schema = Schema({Required('q'): 1})
-    # Can't use nose's raises (because we need to access the raised
-    # exception, nor assert_raises which fails with Python 2.6.9.
+    schema = Schema({Required('q'): int})
+    schema({"q": 123})
     try:
         schema({})
     except Invalid as e:
@@ -68,18 +67,24 @@ def test_iterate_candidates():
 
 def test_in():
     """Verify that In works."""
-    schema = Schema({"color": In(frozenset(["blue", "red", "yellow"]))})
+    schema = Schema({"color": In(frozenset(["red", "blue", "yellow"]))})
     schema({"color": "blue"})
+    try:
+        schema({"color": "orange"})
+    except Invalid as e:
+        assert_equal(str(e), "value must be one of ['blue', 'red', 'yellow'] for dictionary value @ data['color']")
+    else:
+        assert False, "Did not raise InInvalid"
 
 
 def test_not_in():
     """Verify that NotIn works."""
-    schema = Schema({"color": NotIn(frozenset(["blue", "red", "yellow"]))})
+    schema = Schema({"color": NotIn(frozenset(["red", "blue", "yellow"]))})
     schema({"color": "orange"})
     try:
         schema({"color": "blue"})
     except Invalid as e:
-        assert_equal(str(e), "value is not allowed for dictionary value @ data['color']")
+        assert_equal(str(e), "value must not be one of ['blue', 'red', 'yellow'] for dictionary value @ data['color']")
     else:
         assert False, "Did not raise NotInInvalid"
 
@@ -138,7 +143,7 @@ def test_extra_empty_errors():
 
 
 def test_literal():
-    """ test with Literal """
+    """ Test with Literal """
 
     schema = Schema([Literal({"a": 1}), Literal({"b": 1})])
     schema([{"a": 1}])
@@ -197,7 +202,7 @@ def test_class():
 
 
 def test_email_validation():
-    """ test with valid email """
+    """ Test with valid email address """
     schema = Schema({"email": Email()})
     out_ = schema({"email": "example@example.com"})
 
@@ -205,43 +210,43 @@ def test_email_validation():
 
 
 def test_email_validation_with_none():
-    """ test with invalid None Email"""
+    """ Test with invalid None email address """
     schema = Schema({"email": Email()})
     try:
         schema({"email": None})
     except MultipleInvalid as e:
         assert_equal(str(e),
-                     "expected an Email for dictionary value @ data['email']")
+                     "expected an email address for dictionary value @ data['email']")
     else:
-        assert False, "Did not raise Invalid for None url"
+        assert False, "Did not raise Invalid for None URL"
 
 
 def test_email_validation_with_empty_string():
-    """ test with empty string Email"""
+    """ Test with empty string email address"""
     schema = Schema({"email": Email()})
     try:
         schema({"email": ''})
     except MultipleInvalid as e:
         assert_equal(str(e),
-                     "expected an Email for dictionary value @ data['email']")
+                     "expected an email address for dictionary value @ data['email']")
     else:
-        assert False, "Did not raise Invalid for empty string url"
+        assert False, "Did not raise Invalid for empty string URL"
 
 
 def test_email_validation_without_host():
-    """ test with empty host name in email """
+    """ Test with empty host name in email address """
     schema = Schema({"email": Email()})
     try:
         schema({"email": 'a@.com'})
     except MultipleInvalid as e:
         assert_equal(str(e),
-                     "expected an Email for dictionary value @ data['email']")
+                     "expected an email address for dictionary value @ data['email']")
     else:
-        assert False, "Did not raise Invalid for empty string url"
+        assert False, "Did not raise Invalid for empty string URL"
 
 
 def test_fqdn_url_validation():
-    """ test with valid fully qualified domain name url """
+    """ Test with valid fully qualified domain name URL """
     schema = Schema({"url": FqdnUrl()})
     out_ = schema({"url": "http://example.com/"})
 
@@ -249,55 +254,55 @@ def test_fqdn_url_validation():
 
 
 def test_fqdn_url_without_domain_name():
-    """ test with invalid fully qualified domain name url """
+    """ Test with invalid fully qualified domain name URL """
     schema = Schema({"url": FqdnUrl()})
     try:
         schema({"url": "http://localhost/"})
     except MultipleInvalid as e:
         assert_equal(str(e),
-                     "expected a Fully qualified domain name URL for dictionary value @ data['url']")
+                     "expected a fully qualified domain name URL for dictionary value @ data['url']")
     else:
-        assert False, "Did not raise Invalid for None url"
+        assert False, "Did not raise Invalid for None URL"
 
 
 def test_fqdnurl_validation_with_none():
-    """ test with invalid None FQDN url"""
+    """ Test with invalid None FQDN URL """
     schema = Schema({"url": FqdnUrl()})
     try:
         schema({"url": None})
     except MultipleInvalid as e:
         assert_equal(str(e),
-                     "expected a Fully qualified domain name URL for dictionary value @ data['url']")
+                     "expected a fully qualified domain name URL for dictionary value @ data['url']")
     else:
-        assert False, "Did not raise Invalid for None url"
+        assert False, "Did not raise Invalid for None URL"
 
 
 def test_fqdnurl_validation_with_empty_string():
-    """ test with empty string FQDN URL """
+    """ Test with empty string FQDN URL """
     schema = Schema({"url": FqdnUrl()})
     try:
         schema({"url": ''})
     except MultipleInvalid as e:
         assert_equal(str(e),
-                     "expected a Fully qualified domain name URL for dictionary value @ data['url']")
+                     "expected a fully qualified domain name URL for dictionary value @ data['url']")
     else:
-        assert False, "Did not raise Invalid for empty string url"
+        assert False, "Did not raise Invalid for empty string URL"
 
 
 def test_fqdnurl_validation_without_host():
-    """ test with empty host FQDN URL """
+    """ Test with empty host FQDN URL """
     schema = Schema({"url": FqdnUrl()})
     try:
         schema({"url": 'http://'})
     except MultipleInvalid as e:
         assert_equal(str(e),
-                     "expected a Fully qualified domain name URL for dictionary value @ data['url']")
+                     "expected a fully qualified domain name URL for dictionary value @ data['url']")
     else:
-        assert False, "Did not raise Invalid for empty string url"
+        assert False, "Did not raise Invalid for empty string URL"
 
 
 def test_url_validation():
-    """ test with valid URL """
+    """ Test with valid URL """
     schema = Schema({"url": Url()})
     out_ = schema({"url": "http://example.com/"})
 
@@ -305,7 +310,7 @@ def test_url_validation():
 
 
 def test_url_validation_with_none():
-    """ test with invalid None url"""
+    """ Test with invalid None URL"""
     schema = Schema({"url": Url()})
     try:
         schema({"url": None})
@@ -313,11 +318,11 @@ def test_url_validation_with_none():
         assert_equal(str(e),
                      "expected a URL for dictionary value @ data['url']")
     else:
-        assert False, "Did not raise Invalid for None url"
+        assert False, "Did not raise Invalid for None URL"
 
 
 def test_url_validation_with_empty_string():
-    """ test with empty string URL """
+    """ Test with empty string URL """
     schema = Schema({"url": Url()})
     try:
         schema({"url": ''})
@@ -325,11 +330,11 @@ def test_url_validation_with_empty_string():
         assert_equal(str(e),
                      "expected a URL for dictionary value @ data['url']")
     else:
-        assert False, "Did not raise Invalid for empty string url"
+        assert False, "Did not raise Invalid for empty string URL"
 
 
 def test_url_validation_without_host():
-    """ test with empty host URL """
+    """ Test with empty host URL """
     schema = Schema({"url": Url()})
     try:
         schema({"url": 'http://'})
@@ -337,11 +342,11 @@ def test_url_validation_without_host():
         assert_equal(str(e),
                      "expected a URL for dictionary value @ data['url']")
     else:
-        assert False, "Did not raise Invalid for empty string url"
+        assert False, "Did not raise Invalid for empty string URL"
 
 
 def test_copy_dict_undefined():
-    """ test with a copied dictionary """
+    """ Test with a copied dictionary """
     fields = {
         Required("foo"): int
     }
@@ -384,7 +389,6 @@ def test_schema_extend():
 
 def test_schema_extend_overrides():
     """Verify that Schema.extend can override required/extra parameters."""
-
     base = Schema({'a': int}, required=True)
     extended = base.extend({'b': str}, required=False, extra=ALLOW_EXTRA)
 
@@ -396,7 +400,6 @@ def test_schema_extend_overrides():
 
 def test_schema_extend_key_swap():
     """Verify that Schema.extend can replace keys, even when different markers are used"""
-
     base = Schema({Optional('a'): int})
     extension = {Required('a'): int}
     extended = base.extend(extension)
@@ -409,7 +412,6 @@ def test_schema_extend_key_swap():
 
 def test_subschema_extension():
     """Verify that Schema.extend adds and replaces keys in a subschema"""
-
     base = Schema({'a': {'b': int, 'c': float}})
     extension = {'d': str, 'a': {'b': str, 'e': int}}
     extended = base.extend(extension)
@@ -514,7 +516,7 @@ def test_repr():
     )
     assert_equal(repr(coerce_), "Coerce(int, msg='moo')")
     assert_equal(repr(all_), "All('10', Coerce(int, msg=None), msg='all msg')")
-    assert_equal(repr(maybe_int), "Any(None, %s, msg=None)" % str(int))
+    assert_equal(repr(maybe_int), "Any(%s, None, msg=None)" % str(int))
 
 
 def test_list_validation_messages():
@@ -599,6 +601,13 @@ def test_range_outside():
 def test_range_no_upper_limit():
     s = Schema(Range(min=0))
     assert_equal(123, s(123))
+    assert_raises(MultipleInvalid, s, -1)
+
+
+def test_range_no_lower_limit():
+    s = Schema(Range(max=10))
+    assert_equal(-1, s(-1))
+    assert_raises(MultipleInvalid, s, 123)
 
 
 def test_range_excludes_nan():
@@ -631,12 +640,12 @@ def test_clamp_inside():
 
 def test_clamp_above():
     s = Schema(Clamp(min=1, max=10))
-    assert_equal(10, s(12))   
+    assert_equal(10, s(12))
 
 
 def test_clamp_below():
     s = Schema(Clamp(min=1, max=10))
-    assert_equal(1, s(-3)) 
+    assert_equal(1, s(-3))
 
 
 def test_clamp_invalid():
@@ -667,13 +676,13 @@ def test_length_too_short():
 def test_length_too_long():
     v = ['a', 'b', 'c']
     s = Schema(Length(min=0, max=2))
-    assert_raises(MultipleInvalid, s, v) 
+    assert_raises(MultipleInvalid, s, v)
 
 
 def test_length_invalid():
     v = None
     s = Schema(Length(min=0, max=2))
-    assert_raises(MultipleInvalid, s, v) 
+    assert_raises(MultipleInvalid, s, v)
 
 
 def test_equal():
@@ -738,11 +747,73 @@ def test_maybe_accepts_msg():
         assert s([])
 
 
-def test_empty_list_as_exact():
+def test_maybe_returns_subvalidator_error():
+    schema = Schema(Maybe(Range(1, 2)))
+
+    # The following should be valid
+    schema(None)
+    schema(1)
+    schema(2)
+
+    try:
+        # Should trigger a MultipleInvalid exception
+        schema(3)
+    except MultipleInvalid as e:
+        assert_equal(str(e), "value must be at most 2")
+    else:
+        assert False, "Did not raise correct Invalid"
+
+
+def test_schema_empty_list():
     s = Schema([])
-    assert_raises(Invalid, s, [1])
     s([])
 
+    try:
+        s([123])
+    except MultipleInvalid as e:
+        assert_equal(str(e), "not a valid value @ data[123]")
+    else:
+        assert False, "Did not raise correct Invalid"
+
+    try:
+        s({'var': 123})
+    except MultipleInvalid as e:
+        assert_equal(str(e), "expected a list")
+    else:
+        assert False, "Did not raise correct Invalid"    
+
+
+def test_schema_empty_dict():
+    s = Schema({})
+    s({})
+
+    try:
+        s({'var': 123})
+    except MultipleInvalid as e:
+        assert_equal(str(e), "extra keys not allowed @ data['var']")
+    else:
+        assert False, "Did not raise correct Invalid"
+
+    try:
+        s([123])
+    except MultipleInvalid as e:
+        assert_equal(str(e), "expected a dictionary")
+    else:
+        assert False, "Did not raise correct Invalid"
+
+
+def test_schema_empty_dict_key():
+    """ https://github.com/alecthomas/voluptuous/pull/434 """
+    s = Schema({'var': []})
+    s({'var': []})
+
+    try:
+        s({'var': [123]})
+    except MultipleInvalid as e:
+        assert_equal(str(e), "not a valid value for dictionary value @ data['var']")
+    else:
+        assert False, "Did not raise correct Invalid"
+
 
 def test_schema_decorator_match_with_args():
     @validate(int)
@@ -866,7 +937,7 @@ def test_unicode_as_key():
 
 
 def test_number_validation_with_string():
-    """ test with Number with string"""
+    """ Test with Number with string"""
     schema = Schema({"number": Number(precision=6, scale=2)})
     try:
         schema({"number": 'teststr'})
@@ -878,7 +949,7 @@ def test_number_validation_with_string():
 
 
 def test_number_validation_with_invalid_precision_invalid_scale():
-    """ test with Number with invalid precision and scale"""
+    """ Test with Number with invalid precision and scale"""
     schema = Schema({"number": Number(precision=6, scale=2)})
     try:
         schema({"number": '123456.712'})
@@ -890,35 +961,35 @@ def test_number_validation_with_invalid_precision_invalid_scale():
 
 
 def test_number_validation_with_valid_precision_scale_yield_decimal_true():
-    """ test with Number with valid precision and scale"""
+    """ Test with Number with valid precision and scale"""
     schema = Schema({"number": Number(precision=6, scale=2, yield_decimal=True)})
     out_ = schema({"number": '1234.00'})
     assert_equal(float(out_.get("number")), 1234.00)
 
 
 def test_number_when_precision_scale_none_yield_decimal_true():
-    """ test with Number with no precision and scale"""
+    """ Test with Number with no precision and scale"""
     schema = Schema({"number": Number(yield_decimal=True)})
     out_ = schema({"number": '12345678901234'})
     assert_equal(out_.get("number"), 12345678901234)
 
 
 def test_number_when_precision_none_n_valid_scale_case1_yield_decimal_true():
-    """ test with Number with no precision and valid scale case 1"""
+    """ Test with Number with no precision and valid scale case 1"""
     schema = Schema({"number": Number(scale=2, yield_decimal=True)})
     out_ = schema({"number": '123456789.34'})
     assert_equal(float(out_.get("number")), 123456789.34)
 
 
 def test_number_when_precision_none_n_valid_scale_case2_yield_decimal_true():
-    """ test with Number with no precision and valid scale case 2 with zero in decimal part"""
+    """ Test with Number with no precision and valid scale case 2 with zero in decimal part"""
     schema = Schema({"number": Number(scale=2, yield_decimal=True)})
     out_ = schema({"number": '123456789012.00'})
     assert_equal(float(out_.get("number")), 123456789012.00)
 
 
 def test_number_when_precision_none_n_invalid_scale_yield_decimal_true():
-    """ test with Number with no precision and invalid scale"""
+    """ Test with Number with no precision and invalid scale"""
     schema = Schema({"number": Number(scale=2, yield_decimal=True)})
     try:
         schema({"number": '12345678901.234'})
@@ -930,14 +1001,14 @@ def test_number_when_precision_none_n_invalid_scale_yield_decimal_true():
 
 
 def test_number_when_valid_precision_n_scale_none_yield_decimal_true():
-    """ test with Number with no precision and valid scale"""
+    """ Test with Number with no precision and valid scale"""
     schema = Schema({"number": Number(precision=14, yield_decimal=True)})
     out_ = schema({"number": '1234567.8901234'})
     assert_equal(float(out_.get("number")), 1234567.8901234)
 
 
 def test_number_when_invalid_precision_n_scale_none_yield_decimal_true():
-    """ test with Number with no precision and invalid scale"""
+    """ Test with Number with no precision and invalid scale"""
     schema = Schema({"number": Number(precision=14, yield_decimal=True)})
     try:
         schema({"number": '12345674.8901234'})
@@ -949,7 +1020,7 @@ def test_number_when_invalid_precision_n_scale_none_yield_decimal_true():
 
 
 def test_number_validation_with_valid_precision_scale_yield_decimal_false():
-    """ test with Number with valid precision, scale and no yield_decimal"""
+    """ Test with Number with valid precision, scale and no yield_decimal"""
     schema = Schema({"number": Number(precision=6, scale=2, yield_decimal=False)})
     out_ = schema({"number": '1234.00'})
     assert_equal(out_.get("number"), '1234.00')
@@ -1087,9 +1158,9 @@ def test_schema_infer_accepts_kwargs():
 def test_validation_performance():
     """
     This test comes to make sure the validation complexity of dictionaries is done in a linear time.
-    to achieve this a custom marker is used in the scheme that counts each time it is evaluated.
+    To achieve this a custom marker is used in the scheme that counts each time it is evaluated.
     By doing so we can determine if the validation is done in linear complexity.
-    Prior to issue https://github.com/alecthomas/voluptuous/issues/259 this was exponential
+    Prior to issue https://github.com/alecthomas/voluptuous/issues/259 this was exponential.
     """
     num_of_keys = 1000
 
@@ -1443,6 +1514,7 @@ def test_any_required_with_subschema():
     else:
         assert False, "Did not raise Invalid for MultipleInvalid"
 
+
 def test_inclusive():
     schema = Schema({
         Inclusive('x', 'stuff'): int,
@@ -1452,17 +1524,18 @@ def test_inclusive():
     r = schema({})
     assert_equal(r, {})
 
-    r = schema({'x':1, 'y':2})
-    assert_equal(r, {'x':1, 'y':2})
+    r = schema({'x': 1, 'y': 2})
+    assert_equal(r, {'x': 1, 'y': 2})
 
     try:
-        r = schema({'x':1})
+        r = schema({'x': 1})
     except MultipleInvalid as e:
         assert_equal(str(e),
                      "some but not all values in the same group of inclusion 'stuff' @ data[<stuff>]")
     else:
         assert False, "Did not raise Invalid for incomplete Inclusive group"
 
+
 def test_inclusive_defaults():
     schema = Schema({
         Inclusive('x', 'stuff', default=3): int,
@@ -1470,16 +1543,17 @@ def test_inclusive_defaults():
         })
 
     r = schema({})
-    assert_equal(r, {'x':3, 'y':4})
+    assert_equal(r, {'x': 3, 'y': 4})
 
     try:
-        r = schema({'x':1})
+        r = schema({'x': 1})
     except MultipleInvalid as e:
         assert_equal(str(e),
                      "some but not all values in the same group of inclusion 'stuff' @ data[<stuff>]")
     else:
         assert False, "Did not raise Invalid for incomplete Inclusive group with defaults"
 
+
 def test_exclusive():
     schema = Schema({
         Exclusive('x', 'stuff'): int,
@@ -1489,17 +1563,18 @@ def test_exclusive():
     r = schema({})
     assert_equal(r, {})
 
-    r = schema({'x':1})
-    assert_equal(r, {'x':1})
+    r = schema({'x': 1})
+    assert_equal(r, {'x': 1})
 
     try:
-        r = schema({'x':1, 'y': 2})
+        r = schema({'x': 1, 'y': 2})
     except MultipleInvalid as e:
         assert_equal(str(e),
                      "two or more values in the same group of exclusion 'stuff' @ data[<stuff>]")
     else:
         assert False, "Did not raise Invalid for multiple values in Exclusive group"
 
+
 def test_any_with_discriminant():
     schema = Schema({
         'implementation': Union({
@@ -1511,15 +1586,16 @@ def test_any_with_discriminant():
         }, {
             'type': 'C',
             'c-value': bool,
-        }, discriminant=lambda value, alternatives: filter(lambda v : v['type'] == value['type'], alternatives))
+        }, discriminant=lambda value, alternatives: filter(lambda v: v['type'] == value['type'], alternatives))
     })
     try:
         schema({
             'implementation': {
-            'type': 'C',
-            'c-value': None,}  
+                'type': 'C',
+                'c-value': None
+            }
         })
     except MultipleInvalid as e:
-        assert_equal(str(e),'expected bool for dictionary value @ data[\'implementation\'][\'c-value\']')
+        assert_equal(str(e), 'expected bool for dictionary value @ data[\'implementation\'][\'c-value\']')
     else:
         assert False, "Did not raise correct Invalid"
